#ifndef STL_LIST_WRAPPER_H
#define STL_LIST_WRAPPER_H

#include <list>

namespace seq
{
   class STLListWrapper: public Sequence
   {
   public:
      bool empty() const { return this->l.empty(); }
      size_t size() const { return this->l.size(); }
      
      int& front() { return this->l.front(); }
      int& back() { return this->l.back();  }
      
      int& at(int i) { return *this->index(i); }
      std::list<int>::iterator index(int i) {
         auto j = this->l.begin();
         for(int k = 0; k < i; k++) j++;
         return j;
      }

      /* for consistency with own DoublyLinkedList */
      void push(const int& pushed_item) { this->push_back(pushed_item); }

      void push_front(const int& pushed_item) { this->l.push_front(pushed_item); }
      void push_back(const int& pushed_item) { this->l.push_back(pushed_item); }

      /* for consistency with own DoublyLinkedList */
      void pop() { this->pop_front(); }
      
      void pop_front() { this->l.pop_front(); }
      void pop_back() { this->l.pop_back(); }
      void clear() { this->l.clear(); }

      void insert_at(int i, const int& inserted_item) {
         if(i == 0) this->l.push_front(inserted_item);
         else this->l.insert(this->index(i), inserted_item);
      }
      void erase_at(int i) {
         if(i == 0) this->l.pop_front();
         else this->l.erase(this->index(i));
      }
      
      STLListWrapper() {}
      ~STLListWrapper() {}
      STLListWrapper(const STLListWrapper& orig) { this->l = orig.l; }
      STLListWrapper& operator=(const STLListWrapper& rhs) { this->l = rhs.l; return *this; }
      STLListWrapper(STLListWrapper&& old) { this->l = std::move(old.l); }
      STLListWrapper& operator=(STLListWrapper&& old) { this->l = std::move(old.l); return *this; }

   private:
      std::list<int> l = std::list<int>();
   };
}

#endif
